﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Input;

namespace Mandelbrot
{
    public partial class Page : UserControl
    {
        private const int WIDTH = 400;
        private const int HEIGHT = 400;
        private PngEncoder encoder = new PngEncoder(WIDTH, HEIGHT);
        private float cx = -0.75f;
        private float cy = 0.0f;
        private float l = 2.5f;
        private Point p = new Point();
        private bool dragging = false;
        private const int MAXLOOP = 127;
        
        public Page()
        {
            InitializeComponent();
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            Draw();
        }
        private void Draw()
        {
            for (int i = 0; i < HEIGHT; i++)
            {
                int rowStart = encoder.GetRowStart(i);
                for (int j = 0; j < WIDTH; j++)
                {
                    float x = cx + l * ((float)j / (float)WIDTH - 0.5f);
                    float y = cy + l * ((float)i / (float)HEIGHT - 0.5f);
                    Color c = ToColor(Mandelbrot(x, y));
                    encoder.SetPixelAtRowStart(j, rowStart, c.R, c.G, c.B, c.A);
                }
            }
            ImageSource.SetSource(encoder.GetImageStream());
        }

        private static int Mandelbrot(float x, float y)
        {
            float x1 = 0.0f;
            float y1 = 0.0f;
            float x2 = 0.0f;
            float y2 = 0.0f;
            int c = 0;
            for (; c < MAXLOOP; c++)
            {
                float xx = x1 * x1;
                float yy = y1 * y1;
                if (xx + yy > 2.0f) break;
                x2 = xx - yy;
                y2 = x1 * y1 * 2.0f;
                x1 = x2 + x;
                y1 = y2 + y;
            }
            return c;
        }

        private static Color ToColor(int c)
        {
            if (c < 0)
            {
                return Colors.White;
            }
            if (c < MAXLOOP)
            {
                int b = c & 7;
                switch (b)
                {
                    case 0:
                        return Color.FromArgb(255, 128, 255, 0);
                    case 1:
                        return Color.FromArgb(255, 255, 191, 0);
                    case 2:
                        return Color.FromArgb(255, 255, 0, 0);
                    case 3:
                        return Color.FromArgb(255, 255, 0, 191);
                    case 4:
                        return Color.FromArgb(255, 128, 0, 255);
                    case 5:
                        return Color.FromArgb(255, 0, 64, 255);
                    case 6:
                        return Color.FromArgb(255, 0, 255, 255);
                    case 7:
                        return Color.FromArgb(255, 0, 255, 64);
                    default:
                        return Colors.White;
                }
            }
            return Colors.Black;
        }

        private void up_Click(object sender, RoutedEventArgs e)
        {
            l /= 2.0f;
            Draw();
        }

        private void down_Click(object sender, RoutedEventArgs e)
        {
            l *= 2.0f;
            Draw();
        }

        private void img_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            p = e.GetPosition(img);
            dragging = true;
        }

        private void img_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            dragging = false;
            Point mp = e.GetPosition(null);
            float dx = (float)(mp.X - p.X) * l / (float)WIDTH;
            float dy = (float)(mp.Y - p.Y) * l / (float)HEIGHT;
            cx -= dx;
            cy -= dy;
            Draw();
            img.SetValue(Canvas.LeftProperty, 0.0d);
            img.SetValue(Canvas.TopProperty, 0.0d);
        }

        private void img_MouseMove(object sender, MouseEventArgs e)
        {
            if (!dragging) return;
            Point mp = e.GetPosition(null);
            img.SetValue(Canvas.LeftProperty, mp.X - p.X);
            img.SetValue(Canvas.TopProperty, mp.Y - p.Y);
        }
    }
}
